/* Code for test
  (setq c-basic-offset 4)
 */

import java.util.*;
import comp100.*;
import java.io.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.Color;


public class Building{
    private String name;       // name of building
    private int numFloors;     // number of floors in building
    private Warden[] wardens;  // array of warden (or null) on each floor.

    /** Construct a new Building object */
    public Building(String name, int n){
	this.name = name;
	this.numFloors = n;
	this.wardens = new Warden[n];
    }

    /** Returns the name of the building */
    public String getName(){
	return this.name;
    }

    /** Set the warden on the given floor */
    public void setWarden(int floor, Warden w){
	this.wardens[floor] = w;
    }

    /** Return the warden on the given floor */
    public Warden getWarden(int floor){
	return this.wardens[floor];
    }

    /** Print out the name of the building and all the wardens with the floor they are on */
    public void print(){
	System.out.printf("Wardens for %s:\n", this.name);
	for (int fl=0; fl<this.numFloors; fl++){
	    if (this.wardens[fl] != null){
		System.out.printf(" Floor %2d: %s\n", fl, this.wardens[fl]);
	    }
	}
    }

    /** Return the number of floors with no warden */
    public int numMissingWardens(){
	int count = 0;
	for (int fl=0; fl<this.numFloors; fl++){
	    if (this.wardens[fl] == null){
		count = count + 1;
	    }
	}
	return count;
    }

    /** Find the warden closest to the specified floor.
	Prefers wardens on a higher floor rather than a lower floor
	if equally distant. Return null if no wardens in building*/
    public Warden closestWarden1(int fl){
	int upper = fl;
	int lower = fl;
	while(upper < this.numFloors || lower> 0){
	    if (upper < this.numFloors){
		if (this.wardens[upper] != null)
		    return this.wardens[upper];
		upper++;
	    }
	    if (lower > 0){
		if (this.wardens[lower] != null)
		    return this.wardens[lower];
		lower--;
	    }
	}
	return null;
    }

    /** Find the warden closest to the specified floor.
	Prefers wardens on a higher floor rather than a lower floor
	if equally distant. Return null if no wardens in building*/
    public Warden closestWarden2(int fl){
	int upper = fl;
	int lower = fl;
	while (upper<this.numFloors){
	    if (this.wardens[upper] != null)
		break;
	    upper++;
	}
	while (lower>= 0){
	    if (this.wardens[lower] != null)
		break;
	    lower--;
	}
	if (upper >= this.numFloors && lower < 0)
	    return null;
	else if (upper-fl <= fl-lower)
	    return this.wardens[upper];
	else
	    return this.wardens[lower];
    }
    /** Find the warden closest to the specified floor.
	Prefers wardens on a higher floor rather than a lower floor
	if equally distant. Return null if no wardens in building*/
    public Warden closestWarden3(int fl){
	int upper = fl;
	int lower = fl;
	while (upper<this.numFloors && this.wardens[upper] == null){
	    upper++;
	}
	while (lower>= 0 && this.wardens[lower] == null){
	    lower--;
	}
	if (upper >= this.numFloors && lower < 0)
	    return null;
	else if (upper-fl <= fl-lower)
	    return this.wardens[upper];
	else
	    return this.wardens[lower];
    }
    /** Find the warden closest to the specified floor.
	Prefers wardens on a higher floor rather than a lower floor
	if equally distant. Return null if no wardens in building*/
    public Warden closestWarden4(int fl){
	for (int dist = 0; dist<this.numFloors; dist++){
	    if ( fl+dist <this.numFloors  && this.wardens[fl+dist] != null)
		return this.wardens[fl+dist];
	    if ( fl-dist >=0  && this.wardens[fl-dist] != null)
		return this.wardens[fl-dist];
	}
	return null;  // if there were no wardens at all
    }


    // Main
    /** Create a new Building object
     */
    public static void main(String[] arguments){
	Building b = new Building("Hunter", 8);
	b.setWarden(0, new Warden("Pondy", "x1234"));
	b.setWarden(1, new Warden("Lindsay", "x4321"));
	b.setWarden(3, new Warden("John", "x1234"));
	b.setWarden(7, new Warden("Roger", "x1111"));
	test(b);
	b = new Building("Kirk", 4);
	test(b);
    }
	public static void test(Building b){
b.print();
	System.out.printf("There are %d missing wardens\n", b.numMissingWardens());
	for (int i=0; i<b.numFloors; i++)
	    System.out.printf("Closest warden to %d is %s\n", i, b.closestWarden1(i));
	System.out.println();
	for (int i=0; i<b.numFloors; i++)
	    System.out.printf("Closest warden2 to %d is %s\n", i, b.closestWarden2(i));
	System.out.println();
	for (int i=0; i<b.numFloors; i++)
	    System.out.printf("Closest warden3 to %d is %s\n", i, b.closestWarden2(i));
	System.out.println();
	for (int i=0; i<b.numFloors; i++)
	    System.out.printf("Closest warden4 to %d is %s\n", i, b.closestWarden2(i));
	
    }	


}
